function G = grad(mom,x,y,wx,wy,T,sigmaV2,sigmaI2,gammaR,gtabx,gtaby,sqnormy,X1,X2,X3)

% gradient for measure matching algorithm
%
% author : Joan Glaunès - joan.glaunes@free.fr

tau = 1/(T-1);
nx = size(x,2);
ny = size(y,2);
y1 = y(1,:);
y2 = y(2,:);
y3 = y(3,:);

ng = length(gtabx)-1;

mom1 = mom(:,:,1);
mom2 = mom(:,:,2);
mom3 = mom(:,:,3);

if ~exist('X1')
    [X1,X2,X3] = comptraj(x,mom,T,sigmaV2);
end

phiX1 = X1(:,T);
phiX2 = X2(:,T);
phiX3 = X3(:,T);

beta1 = zeros(nx,T-1);
beta2 = zeros(nx,T-1);
beta3 = zeros(nx,T-1);

%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% computes beta at t=1
%%%%%%%%%%%%%%%%%%%%%%%%%%

% computes distance map
for g = 1:ng
    for k = gtabx(g):gtabx(g+1)-1
        beta1loc = 0;
        beta2loc = 0;
        beta3loc = 0;
        for l = gtabx(g):gtabx(g+1)-1
            arg = -( ...
                (phiX1(k)-phiX1(l))^2 + ...
                (phiX2(k)-phiX2(l))^2 + ...
                (phiX3(k)-phiX3(l))^2)/sigmaI2;
            arg = 1/(1-arg)^2;  %% BUILT IN KERNEL derI, do not remove this comment                             
            arg = wx(l) * arg;
            beta1loc = beta1loc + arg * (phiX1(k)-phiX1(l));
            beta2loc = beta2loc + arg * (phiX2(k)-phiX2(l));
            beta3loc = beta3loc + arg * (phiX3(k)-phiX3(l));
        end
        for l = gtaby(g):gtaby(g+1)-1
            arg = -( ...
                (phiX1(k)-y1(l))^2 + ...
                (phiX2(k)-y2(l))^2 + ...
                (phiX3(k)-y3(l))^2)/sigmaI2;
            arg = 1/(1-arg)^2;  %% BUILT IN KERNEL derI, do not remove this comment                             
            arg = - wy(l) * arg;
            beta1loc = beta1loc + arg * (phiX1(k)-y1(l));
            beta2loc = beta2loc + arg * (phiX2(k)-y2(l));
            beta3loc = beta3loc + arg * (phiX3(k)-y3(l));
        end
        loc = (T-2)*nx+k;
        beta1(loc) = - 2 * beta1loc / sigmaI2;
        beta2(loc) = - 2 * beta2loc / sigmaI2;
        beta3(loc) = - 2 * beta3loc / sigmaI2;
    end
end 

%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% computes beta(t)
%%%%%%%%%%%%%%%%%%%%%%%%%%

for t=T-1:-1:2
    
    % computes beta(t) with backward integration
    
    for k = (t-1)*nx+1:t*nx
        beta1loc = 0;
        beta2loc = 0;
        beta3loc = 0;
        for l = (t-1)*nx+1:t*nx
            arg = -((X1(k)-X1(l))^2 + (X2(k)-X2(l))^2 + (X3(k)-X3(l))^2)/sigmaV2(t);
            arg = 1/(1-arg)^2;  %% BUILT IN KERNEL derV, do not remove this comment             
            arg = arg * (beta1(k)*mom1(l) + beta2(k)*mom2(l) + beta3(k)*mom3(l));
            beta1loc = beta1loc + arg * (X1(k)-X1(l));
            beta2loc = beta2loc + arg * (X2(k)-X2(l));
            beta3loc = beta3loc + arg * (X3(k)-X3(l));
        end
        beta1(k-nx) = beta1(k) - 2 * tau * beta1loc / sigmaV2(t);
        beta2(k-nx) = beta2(k) - 2 * tau * beta2loc / sigmaV2(t);
        beta3(k-nx) = beta3(k) - 2 * tau * beta3loc / sigmaV2(t);
    end
end

G = zeros(size(mom));
G(:,:,1) = 2*gammaR*mom1 + beta1;
G(:,:,2) = 2*gammaR*mom2 + beta2;
G(:,:,3) = 2*gammaR*mom3 + beta3;

